/*************************************************************************/
/*                                                                       */
/*  This example program demonstrates the use of the Formatting I/O      */
/*  functions FmtFile and ScanFile to write/read a binary data file.     */
/*  There are four buttons on the Panel, they are Plot, Save Data to     */
/*  Disk, Read Data From Disk, and Quit.  All three buttons use          */
/*  callback functions. The wave generation and plotting are triggered   */
/*  by the Plot button on the front panel, the data is stored to disk by */
/*  the Save Data to Disk button, and the program execution is halted by */
/*  the Quit button.                                                     */
/*                                                                       */
/*************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <formatio.h>
#include <utility.h>
#include <stdlib.h>
#include "fileio.h"

#define COUNT 100
static char proj_dir[MAX_PATHNAME_LEN];
static char file_name[MAX_PATHNAME_LEN];
static double wave[100];
static int handle;
static int err;
static char temp[40];

error_check(char *temp, int err);

/*************************************************************************/
/*                                                                       */
/*  The main function is very simple when you use callbacks.  Notice     */
/*  that you only have to load and display the panels. The callback      */
/*  functions are installed when you load the panels.  Calling           */
/*  RunUserInterface simply tells LabWindows/CVI to begin passing all    */
/*  event information from the front panel to the appropriate callback   */
/*  functions.                                                           */
/*                                                                       */
/*************************************************************************/

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    GetProjectDir (proj_dir);
    handle = LoadPanel (0, "fileio.uir", Examp1);
    SetPanelAttribute(handle, ATTR_TITLE, "Example - Write to Ascii File");
    DisplayPanel (handle);
    RunUserInterface ();

    return 0;
}

/*************************************************************************/
/*                                                                       */
/*  Open the fileio.uir file to see how the callback functions are       */
/*  assigned.  If you edit the Plot button in the UIR editor, you        */
/*  will see the Plot function name listed in the Callback Function      */
/*  control in the dialog box.                                           */
/*                                                                       */
/*  The Plot function is assigned to the Plot Data button in the  User   */
/*  Interface Editor.  The function is automatically prototyped in the   */
/*  header file fileio.h generated by LabWindows/CVI.  This function is  */
/*  called by LabWindows/CVI when you click on the Plot Data button      */
/*  during program execution.                                            */
/*                                                                       */
/*  Notice that you must decide which event to respond to in the func-   */
/*  tion.  In this case, the function is executed only when a COMMIT     */
/*  event is sensed (up mouse click).  LabWindows/CVI can recognize      */
/*  and pass many different event types, such as LEFT_CLICK, RIGHT_CLICK,*/
/*  LEFT_DOUBLE_CLICK, and so on.  To find out more about these events,  */
/*  refer to the event.prj example in the samples\userint directory.     */
/*                                                                       */
/*************************************************************************/

int CVICALLBACK Plot(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int i;

    if (event == EVENT_COMMIT)
    {
        for (i=0;i<COUNT;i++)
        wave[i] = (double)rand();

        DeleteGraphPlot (handle, Examp1_Graph, -1, 1);
        PlotY (handle, Examp1_Graph, wave, COUNT, VAL_DOUBLE, VAL_THIN_LINE,
               VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
        SetCtrlAttribute (handle, Examp1_Save, ATTR_DIMMED, 0);
    }
    return(0);
}

/*************************************************************************/
/*                                                                       */
/*  The Save function is assigned to the Save Data to Disk button in the */
/*  User Interface Editor.  The function is automatically prototyped in  */
/*  the header file fileio.h generated by LabWindows/CVI.  This function */
/*  is called by LabWindows/CVI when you click on the Save Data button   */
/*  during program execution.  This function brings up a File Selection  */
/*  Box and allows you to enter a file name with a .dat extension.       */
/*  After the file name is entered or selected that file is opened as an */
/*  ASCII file and assigned a file handle.  This file handle is used     */
/*  anytime action is performed on the file.  The file is then written   */
/*  using the FmtFile function.  The file is then closed.                */
/*                                                                       */
/*************************************************************************/

int CVICALLBACK Save(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{

    int i, fileType;

    if (event == EVENT_COMMIT){
        if (FileSelectPopup (proj_dir, "*.dat", "*.dat", "Name of File to Save",
              VAL_OK_BUTTON, 0, 1, 1, 0, file_name) > 0)
        {
            GetCtrlVal (handle, Examp1_OutputType, &fileType);
            ArrayToFile (file_name, wave, VAL_DOUBLE, COUNT, 1, VAL_GROUPS_TOGETHER,
                 VAL_GROUPS_AS_COLUMNS, VAL_CONST_WIDTH, 10, fileType,
                 VAL_TRUNCATE);
            SetCtrlAttribute (handle, Examp1_Save, ATTR_DIMMED, 1);
            SetCtrlAttribute (handle, Examp1_Read, ATTR_DIMMED, 0);
        }
    }
    return(0);
}


/*************************************************************************/
/*                                                                       */
/*  Open the fileio.uir file to see how the callback functions are       */
/*  assigned.  If you edit the Read Data button in the UIR editor, you   */
/*  will see the Read function name listed in the Callback Function      */
/*  control in the dialog box.                                           */
/*                                                                       */
/*  The Read function is assigned to the Read Data button in the  User   */
/*  Interface Editor.  The function is automatically prototyped in the   */
/*  header file fileio.h generated by LabWindows/CVI.  This function is  */
/*  called by LabWindows/CVI you click on the Read Data button during    */
/*  program execution.                                                   */
/*                                                                       */
/*  Notice that you must decide which event to respond to in the func-   */
/*  tion.  In this case, the function is executed only when a COMMIT     */
/*  event is sensed (up mouse click).  LabWindows/CVI can recognize      */
/*  and pass many different event types, such as LEFT_CLICK, RIGHT_CLICK,*/
/*  LEFT_DOUBLE_CLICK, and so on.  To find out more about these events,  */
/*  refer to the event.prj example in the samples\userint directory.     */
/*                                                                       */
/*************************************************************************/

int CVICALLBACK Read(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int i;
    int status, fileType;

    if (event == EVENT_COMMIT){
        if (FileSelectPopup ("", "*.dat", "*.dat", "Name of File to Read",
              VAL_OK_BUTTON, 0, 1, 1, 0, file_name) > 0)
        {
            GetCtrlVal (handle, Examp1_InputType, &fileType);
            FileToArray (file_name, wave, VAL_DOUBLE, COUNT, 1,
                 VAL_GROUPS_TOGETHER, VAL_GROUPS_AS_COLUMNS, fileType);
            DeleteGraphPlot (handle, Examp1_Graph2, -1, 1);
            err = PlotY (handle, Examp1_Graph2, wave, COUNT, VAL_DOUBLE,
                  VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1,VAL_RED);
        }
    }
    return (0);
}



/*************************************************************************/
/*                                                                       */
/*  The Quit function is assigned to the Quit button in the User         */
/*  Interface Editor.  The function is automatically prototyped in the   */
/*  header file wrtfile.h generated by LabWindows/CVI.  This function is */
/*  called by LabWindows/CVI when you click on the Quit button during    */
/*  program execution.  Calling QuitUserInterface simply tells           */
/*  LabWindows/CVI to stop passing all event information from the front  */
/*  panel.  This will terminate the program .                            */
/*                                                                       */
/*************************************************************************/

int CVICALLBACK Quit(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        QuitUserInterface(0);
    return(0);
}

