/******************************************************************************/
/* This program illustrates the use of the PRTHIRES instrument to print       */
/* graphs and strip charts in better resolution. Feel free to modify the .UIR */
/* file and experiment with different styles of graphs and strip charts. You  */
/* can either print them with PrintCtrl() User Interface Library function, or */
/* with the PrintGraphHiRes() PRTHIRES instrument function.                   */
/******************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <ansi_c.h>
#include "prthires.h"
#include "hires.h"

#define MAX_FONT_PAIRS  10          /* max. size of font substitution entries */
#define MAXLEN          256         /* max. length of any string */

#define kXResolution    300         /* in dots/inch */
#define kYResolution    300         /* in dots/inch */
#define kNumPoints      100         /* no. of points to plot */

static char *fontSubstitutionTable[MAX_FONT_PAIRS*2] = {0};

static void SetPrintAttributes (void);
static void FillFontSubstitutionTable (char **fontSubstitutionTable);
static void GetAndPlotDataOnTheGraphOrStripChart (int panel, int controlId);
static void DiscardFontSubstitutionTable (char **fontSubstitutionTable);

int main (int argc, char *argv[])
{
    int panel;

    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    panel = LoadPanel (0, "hires.uir", PANEL);
    DisplayPanel (panel);

    SetPrintAttributes();
    FillFontSubstitutionTable (fontSubstitutionTable);
    GetAndPlotDataOnTheGraphOrStripChart (panel, PANEL_CONTROL);

    SetPanelAttribute (panel, ATTR_CLOSE_CTRL, PANEL_QUIT);
    RunUserInterface();
                                                            /* ... and after */
    DiscardFontSubstitutionTable (fontSubstitutionTable);
    DiscardPanel (panel);

    return 0;
}

/***************************************************************************/

static void SetPrintAttributes (void)
{
    int paperHeight, paperWidth;
    int orientation;

    SetPrintAttribute (ATTR_PAPER_HEIGHT, -1);
    SetPrintAttribute (ATTR_PAPER_WIDTH, 0);
    SetPrintAttribute (ATTR_XRESOLUTION, kXResolution);
    SetPrintAttribute (ATTR_YRESOLUTION, kYResolution);
    SetPrintAttribute (ATTR_COLOR_MODE, VAL_BW);
    SetPrintAttribute (ATTR_ORIENTATION, VAL_LANDSCAPE);

    GetPrintAttribute (ATTR_PAPER_HEIGHT, &paperHeight);
    GetPrintAttribute (ATTR_PAPER_WIDTH, &paperWidth);
}

/***************************************************************************/

static void FillFontSubstitutionTable (char **table)
{
    int i = 0;

    for (i = 0; i < 4; i++)
        table[i] = calloc (MAXLEN, sizeof (char));

/***************************************************************************/
/* Attention UNIX users: On some X servers the fonts chosen may not be     */
/* scalable, which may cause the resulting text to appear too small or to  */
/* appear blockish. If that is the case, then you should use this table to */
/* switch to a font which you know is scalable on your system.             */
/***************************************************************************/

    strcpy (table [0], "System");
    strcpy (table [1], "Times NEW Roman");
    strcpy (table [2], "Times NEW Roman");
    strcpy (table [3], "NIDialog");
    table [4] = NULL;

    return;
}

/***************************************************************************/

static void GetAndPlotDataOnTheGraphOrStripChart (int panel, int controlId)
{
    double  dataPoints [kNumPoints];
    double  xArray [kNumPoints], yArray [kNumPoints];
    int     i;

    for (i = 0;i < kNumPoints; i++)
        xArray[i] = rand() / 32767.0 * 100.0;
    for (i = 0; i < kNumPoints; i++)
        yArray[i] = rand() / 32767.0 * 100.0;

    for (i = 0; i < kNumPoints; i++)
        dataPoints[i] = rand() /32767.0 * 100.0;

    PlotStripChart (panel, controlId, dataPoints, kNumPoints, 0, 0, VAL_DOUBLE);

#if 0
    PlotOval (panel, controlId, 20, 20, 200, 200, VAL_RED, VAL_RED);
    PlotText (panel, controlId, 100, 100, "This is just a", VAL_APP_META_FONT,
        VAL_RED, VAL_TRANSPARENT);
    PlotText (panel, controlId, 150, 150, "demonstration of how", VAL_APP_META_FONT,
        VAL_RED, VAL_TRANSPARENT);
    PlotText (panel, controlId, 200, 200, "text plots work...", VAL_APP_META_FONT,
        VAL_RED, VAL_TRANSPARENT);
    PlotArc (panel, controlId, 20, 20, 150, 150, 45, 500, VAL_MAGENTA,
        VAL_TRANSPARENT);
    PlotRectangle (panel, controlId, 50, 50, 150, 150, VAL_RED,
        VAL_RED);
    PlotPoint (panel, controlId, 10, 10, VAL_X, VAL_RED);
    PlotWaveform (panel, controlId, dataPoints, kNumPoints, VAL_DOUBLE, 1.0, 0.0,
        0.0, 1.0, VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_DASH_DOT_DOT,1, VAL_RED);
    PlotPolygon (panel, controlId, xArray, yArray, kNumPoints, VAL_DOUBLE,
        VAL_DOUBLE, VAL_RED, VAL_RED);
    PlotXY (panel, controlId, xArray, yArray, kNumPoints, VAL_DOUBLE,
        VAL_DOUBLE, VAL_VERTICAL_BAR, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
    PlotY (panel, controlId, dataPoints, kNumPoints, VAL_DOUBLE, VAL_THIN_LINE,
        VAL_DOTTED_EMPTY_CIRCLE, VAL_SOLID, 1, VAL_RED);
    PlotX (panel, controlId, dataPoints, kNumPoints, VAL_DOUBLE, VAL_THIN_LINE,
        VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
#endif

    return;
}

/***************************************************************************/

static void DiscardFontSubstitutionTable (char **fontSubstitutionTable)
{
    int i;

    for (i = 0; fontSubstitutionTable[i] != NULL; i++)
        free (fontSubstitutionTable[i]);
}

/***************************************************************************/

int CVICALLBACK PrintLoRes (int panel, int control, int event, void *callbackData,
    int eventData1, int eventData2)
{
    static int  i = 0;
    
    if (event == EVENT_COMMIT)
        PrintCtrl (panel, PANEL_CONTROL, "", 1, 0);

    return 0;
}

/***************************************************************************/

int CVICALLBACK PrintHiRes (int panel, int control, int event, void *callbackData,
    int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        PrintGraphHiRes (panel, PANEL_CONTROL, "", fontSubstitutionTable);

    return 0;
}

/***************************************************************************/

int CVICALLBACK Quit (int panel, int control, int event, void *callbackData, int eventData1,
    int eventData2)
{
    if (event == EVENT_COMMIT)
        QuitUserInterface (0);

    return 0;
}
