/* Dual Y axis Example Program */

#define HELPTEXT "\
This sample program illustrates the use of a left and right Y axis on the\n\
same graph.  Use the Plot Data button to draw the data on the graph.  The\n\
red graph cursor is associated with the left axis while the blue cursor is\n\
associated with the right axis.  The pairs of numeric indicators display the\n\
current values of the graph cursors.  The Show Bitmap checkbox illustrates\n\
the use of a transparent background color for the graph."

#include <cvirte.h>     /* needed if linking executable in external compiler; harmless otherwise */
#include <userint.h>
#include <math.h>
#include "2yaxis.h"

#define NBLEFT  75
#define NBRIGHT 100

static int panelHandle;

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    if ((panelHandle = LoadPanel (0, "2yaxis.uir", PANEL)) < 0)
        return -1;
    SetCtrlAttribute (panelHandle, PANEL_PICTURE, ATTR_VISIBLE, 0);
    DisplayPanel (panelHandle);
    RunUserInterface ();

    return 0;
}

int CVICALLBACK Quit (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2) {
    switch (event) {
        case EVENT_COMMIT:
            QuitUserInterface (0);
        break;
    }
    return 0;
}

int CVICALLBACK GenerateData (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2) {
    
    int         i;
    double  TabLeft[NBLEFT];
    double  TabRight[NBRIGHT];
    
    switch (event) {
        case EVENT_COMMIT:
            /* Plot the data for the left axis */
            for (i=0; i<NBLEFT; i++){
                TabLeft[i]= 1.0 + i*0.063 - i*i*0.00066;
            }
            SetCtrlAttribute (panelHandle, PANEL_GRAPH, ATTR_ACTIVE_YAXIS, VAL_LEFT_YAXIS);
            PlotY (panelHandle, PANEL_GRAPH, TabLeft, NBLEFT, VAL_DOUBLE,
                         VAL_FAT_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
        
            
            /* Plot the data for the right axis */
            for (i=0; i<NBRIGHT; i++){
                TabRight[i]=10000* exp(-i*0.05);
            }
            SetCtrlAttribute (panelHandle, PANEL_GRAPH, ATTR_ACTIVE_YAXIS, VAL_RIGHT_YAXIS);
            PlotY (panelHandle, PANEL_GRAPH, TabRight, NBRIGHT, VAL_DOUBLE, VAL_FAT_LINE,
                         VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_BLUE);
            
            SetCtrlAttribute (panelHandle, PANEL_DATA, ATTR_DIMMED, 1);
            
            /* Initialize the numeric indicators by forcing an active graph cursor */
            /* and calling the Graph function to initialize the numerics */
            SetActiveGraphCursor (panelHandle, PANEL_GRAPH, 1);
            Graph (0, 0, EVENT_VAL_CHANGED,0, 0, 0);
            SetActiveGraphCursor (panelHandle, PANEL_GRAPH, 2);
            Graph (0, 0, EVENT_VAL_CHANGED,0, 0, 0);
            
            SetActiveCtrl (panelHandle, PANEL_GRAPH);
        break;
    }
    return 0;
}

int CVICALLBACK Graph (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2) {
    
    int         Cursor;
    double  X, Y;
    
    switch (event) {
        case EVENT_VAL_CHANGED:
            GetActiveGraphCursor (panelHandle, PANEL_GRAPH, &Cursor);
            GetGraphCursor (panelHandle, PANEL_GRAPH, Cursor, &X, &Y);
            if(Cursor==1){
                SetCtrlVal(panelHandle, PANEL_XL, X);
                SetCtrlVal(panelHandle, PANEL_YL, Y);
            }else{
                SetCtrlVal(panelHandle, PANEL_XR, X);
                SetCtrlVal(panelHandle, PANEL_YR, Y);
            }
        break;
    }
    return 0;
}

int CVICALLBACK ScaleIt (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2) {
    
    int Val;
    
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal(panelHandle, control, &Val);
            SetCtrlAttribute (panelHandle, PANEL_GRAPH, ATTR_ACTIVE_YAXIS, VAL_RIGHT_YAXIS);   
            SetCtrlAttribute (panelHandle, PANEL_GRAPH, ATTR_YMAP_MODE, Val);
            SetActiveCtrl (panelHandle, PANEL_GRAPH);
        break;
    }
    return 0;
}

int CVICALLBACK ToggleBitmap (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int value;
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle, PANEL_CHECKBOX, &value);
            SetCtrlAttribute (panelHandle, PANEL_PICTURE, ATTR_VISIBLE, value);
            break;
    }
    return 0;
}

int CVICALLBACK Help (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            MessagePopup ("Dual Y axis Example Program", HELPTEXT);
            break;
    }
    return 0;
}
