/* Graph and Canvas Benchmark Sample Program */

#define HELPTEXT "\
This sample program illustrates the speed performance gain when using\n\
a Canvas control instead of a Graph Control for arbitrary drawing.\n\n\
Plot the Graph first and then the Canvas to see the performance ratio\n\
displayed in the numeric control."

#include <cvirte.h>     /* needed if linking executable in external compiler; harmless otherwise */
#include <utility.h>
#include <ansi_c.h>  
#include <userint.h>
#include "canvsbmk.h"

static int numLines, linesDrawn=0;
static int canvasWidth;
static int canvasHeight;
static int useGraph;
static int xPrev=0, yPrev=0, xPres, yPres, randColor;
static double time0, time1;
static double graphtime, canvastime;
static int useColor;

static int panelHandle;

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    if ((panelHandle = LoadPanel (0, "canvsbmk.uir", PANEL)) < 0)
        return -1;

    /* Setup the user interface */
    GetCtrlAttribute (panelHandle, PANEL_CANVAS, ATTR_HEIGHT, &canvasHeight);
    GetCtrlAttribute (panelHandle, PANEL_CANVAS, ATTR_WIDTH, &canvasWidth);
    SetAxisRange (panelHandle, PANEL_GRAPH, VAL_MANUAL, 0.0, canvasWidth,
                  VAL_MANUAL, 0.0, canvasHeight);
    DisplayPanel (panelHandle);
    RunUserInterface ();
    
    return 0;
}

int CVICALLBACK PlotGraph (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            /* Get the number of lines to plot */
            GetCtrlVal (panelHandle, PANEL_NUMERIC, &numLines);
            
            /* Loop to plot each line */
            time0 = Timer();
            for (linesDrawn=0;linesDrawn<numLines;linesDrawn++) {
                xPres = rand()%(canvasWidth+1);
                yPres = rand()%(canvasHeight+1);
                if (useColor)
                    randColor = MakeColor(rand()%256, rand()%256, rand()%256);
                PlotLine (panelHandle, PANEL_GRAPH, xPrev, yPrev, xPres, yPres,
                              randColor);
                xPrev = xPres;
                yPrev = yPres;
            }
            time1 = Timer();
            
            /* Display the speed in seconds */
            graphtime = time1-time0;
            SetCtrlVal (panelHandle, PANEL_TimeGraph, graphtime);
            SetCtrlVal (panelHandle, PANEL_Difference, 0.0);
            break;
    }
    return 0;
}

int CVICALLBACK PlotCanvas (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            /* Get the number of lines to plot */
            GetCtrlVal (panelHandle, PANEL_NUMERIC, &numLines);
            
            /* Loop to plot each line inside the start and end of the batch draw functions */
            time0 = Timer();
            CanvasStartBatchDraw (panelHandle, PANEL_CANVAS);
            for (linesDrawn=0;linesDrawn<numLines;linesDrawn++) {
                xPres = rand()%(canvasWidth+1);
                yPres = rand()%(canvasHeight+1);
                if (useColor) {
                    randColor = MakeColor(rand()%256, rand()%256, rand()%256);
                    SetCtrlAttribute (panelHandle, PANEL_CANVAS, ATTR_PEN_COLOR,
                                      randColor);
                    }
                CanvasDrawLineTo (panelHandle, PANEL_CANVAS, MakePoint(xPres, yPres));
                }
            CanvasEndBatchDraw (panelHandle, PANEL_CANVAS);
            time1 = Timer();
            
            /* Display the speed in seconds */
            canvastime = time1-time0;
            SetCtrlVal (panelHandle, PANEL_TimeCanvas, canvastime);
            SetCtrlVal (panelHandle, PANEL_Difference, graphtime/canvastime);
            break;
    }
    return 0;
}

int CVICALLBACK Quit (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            QuitUserInterface (0);
            break;
    }
    return 0;
}

int CVICALLBACK ClearBoth (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            DeleteGraphPlot (panelHandle, PANEL_GRAPH, -1, VAL_IMMEDIATE_DRAW);
            CanvasClear (panelHandle, PANEL_CANVAS, VAL_ENTIRE_OBJECT);
            break;
    }
    return 0;
}

int CVICALLBACK UseColor (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle, PANEL_BINARYSWITCH, &useColor);
            break;
    }
    return 0;
}

int CVICALLBACK Help (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            MessagePopup ("Graph and Canvas Benchmark", HELPTEXT);

            break;
    }
    return 0;
}
