
#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <ansi_c.h>
#include <userint.h>
#include "toolbox.h"    /* for Random() and TWO_PI */
#include "clipbord.h"

static int panelHandle;
void CVICALLBACK MenuDimmerCallback (int menuBarHandle, int panel);
  
  
int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    panelHandle = LoadPanel (0, "clipbord.uir", PANEL);
    InstallMenuDimmerCallback ( GetPanelMenuBar(panelHandle), MenuDimmerCallback);
    DisplayPanel (panelHandle);
    RunUserInterface ();

    return 0;
}

    /* Dim the menu items appropriately when the menubar is clicked on */
void CVICALLBACK MenuDimmerCallback (int menuBarHandle, int panel)
{
    int textAvailable, bitmapAvailable, activeCtrl;
    
    ClipboardGetText(0, &textAvailable);        /* check if text is available on the clipboard */
    ClipboardGetBitmap(0, &bitmapAvailable);    /* check if a bitmap is available on the clipboard */
    activeCtrl = GetActiveCtrl(panel);          

    SetMenuBarAttribute (menuBarHandle, BAR_COPY_PASTE, ATTR_DIMMED, !textAvailable && !bitmapAvailable);
    SetMenuBarAttribute (menuBarHandle, BAR_COPY_COPYACTIVE, ATTR_DIMMED, activeCtrl <= 0);
}

    /* Handle Exit menu item */
void CVICALLBACK ExitCallback (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    QuitUserInterface (0);
}

    /* Handle close box on panel */
int CVICALLBACK PanelCallback (int panel, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_CLOSE)
        QuitUserInterface (0);

    return 0;
}

    /* Copy all of the text in the "Text to Copy" text box */
void CVICALLBACK CopyTextCallback (int menuBar, int menuItem, void *callbackData, int panel)
{
    int     length;
    char    *text;
    
    GetCtrlAttribute(panel, PANEL_TEXT_TO_COPY, ATTR_STRING_TEXT_LENGTH, &length);
    length = length + 1;    /* add space for the nul terminating byte at the end of the string */
    
    /* allocate a buffer to copy the text in the text box control into */
    text = (char *)malloc(length);      
    if (text != NULL)
        {
        GetCtrlVal(panel, PANEL_TEXT_TO_COPY, text);    /* put the text in the buffer */
        ClipboardPutText(text);             /* put the text in the buffer on the clipboard */
        free(text);                         /* free the buffer */
        }
}

void CVICALLBACK CopyActiveCtrlCallback (int menuBar, int menuItem, void *callbackData, int panel)
{
    int ctrl;
    int bitmap;
    int includeLabel;
    
    ctrl = GetActiveCtrl(panel);
    if (ctrl > 0)
        {
        includeLabel = ConfirmPopup("Copying Active Control to Clipboard", "Include label?");
        GetCtrlDisplayBitmap(panel, ctrl, includeLabel, &bitmap); 
        ClipboardPutBitmap(bitmap);                   
        DiscardBitmap(bitmap);      
        }
}

void CVICALLBACK CopyPanelImageCallback (int menuBar, int menuItem, void *callbackData, int panel)
{
    int bitmap;
    
    GetPanelDisplayBitmap(panel, VAL_VISIBLE_AREA, VAL_ENTIRE_OBJECT, &bitmap); 
    ClipboardPutBitmap(bitmap);                   
    DiscardBitmap(bitmap);      
}

void CVICALLBACK PasteCallback (int menuBar, int menuItem, void *callbackData, int panel)
{
    char    *text = 0;
    int     bitmap = 0;
    Rect    destRect;
    
    ClipboardGetText(&text, 0);    
    if (text != 0)      /* any text to paste? */
        {
        ResetTextBox(panel, PANEL_PASTED_TEXT, text); 
        free(text);     /* free the copy of the text returned by ClipboardGetText */
        }
    else
        {
        ClipboardGetBitmap(&bitmap, 0);  
        if (bitmap != 0)    /* is there a bitmap to paste? */
            {
            CanvasClear(panel, PANEL_CANVAS, VAL_ENTIRE_OBJECT);
            
                /* ask user how to draw the clipboard's bitmap into the canvas control: */
            if (GenericMessagePopup ("Pasting Clipboard Image To Canvas",
                        "How should the image on the clipboard be pasted into the canvas control?",
                        "__Preserve Image Size", "__Stretch To Fit The Canvas", 0, 0, 0,
                        0, VAL_GENERIC_POPUP_BTN1, VAL_GENERIC_POPUP_BTN1, VAL_GENERIC_POPUP_BTN2)
                    == VAL_GENERIC_POPUP_BTN1)  
                destRect = MakeRect(0,0,VAL_KEEP_SAME_SIZE, VAL_KEEP_SAME_SIZE);
            else
                destRect = VAL_ENTIRE_OBJECT;
                    
            CanvasDrawBitmap (panel, PANEL_CANVAS, bitmap, VAL_ENTIRE_OBJECT, destRect);
            DiscardBitmap(bitmap);  /* free the bitmap returned by ClipboardGetBitmap */
            }
        }
}

void CVICALLBACK AboutBoxCallback (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    int aboutBox;
    
    aboutBox = LoadPanel(0, "clipbord.uir", ABOUT_BOX);
    InstallPopup(aboutBox);
    GetUserEvent(1,0,0);    /* wait for a button press */
    DiscardPanel(aboutBox);
}

    /*  This function plots some points on the graph so that it will look more interesting when copied to the clipboard */
int CVICALLBACK GraphUpdateCallback (int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    #define NUM_POINTS  50

    static  double      x[NUM_POINTS], y[NUM_POINTS];
    static  int         initialized = 0;
    int                 index;

    switch (event)
        {
        case EVENT_TIMER_TICK:
            if (!initialized)
                {
                initialized = 1;
                for (index = 0; index < NUM_POINTS; index++)        /* make a circle */
                    {
                    x[index] = cos(TWO_PI * index / (NUM_POINTS - 1));
                    y[index] = sin(TWO_PI * index / (NUM_POINTS - 1));
                    }
                }

            for (index = 0; index < NUM_POINTS; index++)    /* randomize the circle a little bit every tick */
                {
                x[index] += Random(-.02, .02) + .02;    
                y[index] += Random(-.02, .02) + .01;
                }
            
            DeleteGraphPlot (panel, PANEL_GRAPH, -1, VAL_DELAYED_DRAW);
            PlotPolygon (panel, PANEL_GRAPH, x, y, NUM_POINTS,
                         VAL_DOUBLE, VAL_DOUBLE, VAL_RED, VAL_BLACK);
            break;
        }
    return 0;
}
