/*********************************************************************************************/
/* This sample program demonstrates a possible use of the GetImageBits() and SetImageBits()  */
/* functions. In this case, they are used to edit an existing image, by selectively altering */
/* colors from the color table. You can use your own images in the picture ring provided. If */
/* you have a hard time finding any, there should always be a few .bmp files in your Windows */
/* directory that you can use.                                                               */
/*********************************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <utility.h>
#include <userint.h>
#include <ansi_c.h>
#include "imagedit.h"

static void UpdateImageControls (void);

static int  panelHandle;


int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    panelHandle = LoadPanel (0, "imagedit.uir", PANEL);
    DisplayPanel (panelHandle);
    RunUserInterface();

    return 0;
}


int CVICALLBACK ColorTableIndexCallback (int panel, int control, int event, void *callbackData,
    int eventData1, int eventData2)
{
    unsigned char   *bitmap, *mask;
    int             colorTableIndex, color, colorTableSize, bitmapSize, maskSize, index,
                    *colorTable;
    
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlIndex (panel, PANEL_PICTURE, &index);
            AllocImageBits (panel, PANEL_PICTURE, index, &colorTable, &bitmap, &mask); 
            GetImageBits (panel, PANEL_PICTURE, index, NULL, NULL, NULL, NULL,
                colorTable, bitmap, mask);
            GetCtrlVal (panel, PANEL_COLORTABLEINDEX, &colorTableIndex);
            color = colorTable[colorTableIndex];
            SetCtrlVal (panel, PANEL_COLORTABLEENTRY, color);
            free (colorTable);
            free (bitmap);
            free (mask);
            break;
        }
    return 0;
}


int CVICALLBACK PictureRingCallback (int panel, int control, int event, void *callbackData, int eventData1,
    int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            UpdateImageControls();
            break;
        }
    return 0;
}


int CVICALLBACK QuitCallback (int panel, int control, int event, void *callbackData, int eventData1,
    int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            QuitUserInterface (0);
            break;
        }
    return 0;
}


int CVICALLBACK LoadImageCallback (int panel, int control, int event, void *callbackData, int eventData1,
    int eventData2)
{
    int     retVal, index, numItems;
    char    imageFileBuf[MAX_PATHNAME_LEN];
    
    switch (event) {
        case EVENT_COMMIT:
            retVal = FileSelectPopup ("", "*.bmp", "*.pcx;*.bmp;*.ico;*.rle;*.dib",
                "Load Image File", VAL_LOAD_BUTTON, 0, 0, 1, 0, imageFileBuf);
            if (retVal == VAL_EXISTING_FILE_SELECTED) {
                InsertListItem (panel, PANEL_PICTURE, -1, imageFileBuf, 0); 
                GetNumListItems (panel, PANEL_PICTURE, &numItems);
                index = numItems - 1;
                SetCtrlIndex (panel, PANEL_PICTURE, index);
                SetCtrlAttribute (panel, PANEL_UPDATECOLORTABLE, ATTR_DIMMED, 0);
                UpdateImageControls();
                }
            break;
        }
    return 0;
}


int CVICALLBACK UpdateColorTableCallback (int panel, int control, int event, void *callbackData,
    int eventData1, int eventData2)
{
    unsigned char   *bitmap, *mask;
    int             colorTableIndex, color, index, rowBytes, depth, width, height,
                    colorTableSize, bitmapSize, maskSize, *colorTable;
    
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlIndex (panel, PANEL_PICTURE, &index);
            AllocImageBits (panel, PANEL_PICTURE, index, &colorTable, &bitmap, &mask); 
            GetImageBits (panel, PANEL_PICTURE, index, &rowBytes, &depth, &width, &height,
                colorTable, bitmap, mask);
            GetCtrlVal (panel, PANEL_COLORTABLEINDEX, &colorTableIndex);
            GetCtrlVal (panel, PANEL_COLORTABLEENTRY, &color);
            colorTable[colorTableIndex] = color;
            SetImageBits (panel, PANEL_PICTURE, index, rowBytes, depth, width, height,
                colorTable, bitmap, mask);
            free (colorTable);
            free (bitmap);
            free (mask);
            break;
        }
    return 0;
}


static void UpdateImageControls (void)
{
    unsigned char   istr[7], *bitmap, *mask;
    int             rowBytes, depth, width, height, panel, index, colorTableSize, bitmapSize,
                    maskSize, i, numEntries, *colorTable;

    panel = panelHandle;
    GetCtrlIndex (panel, PANEL_PICTURE, &index);
    if (index == -1)
        return;
    GetImageInfo (panel, PANEL_PICTURE, index, &colorTableSize, &bitmapSize, &maskSize);
    colorTable = malloc (colorTableSize);
    bitmap = malloc (bitmapSize);
    mask = malloc (maskSize);
    GetImageBits (panel, PANEL_PICTURE, index, &rowBytes, &depth, &width, &height,
        colorTable, bitmap, mask);
        
    SetCtrlAttribute (panelHandle, PANEL_COLORTABLEINDEX, ATTR_DIMMED, 0);
    SetCtrlAttribute (panelHandle, PANEL_COLORTABLEENTRY, ATTR_DIMMED, 0);
    SetCtrlAttribute (panelHandle, PANEL_UPDATECOLORTABLE, ATTR_DIMMED, 0);
    SetCtrlAttribute (panelHandle, PANEL_TEXTMSG_2, ATTR_DIMMED, 0);

    SetCtrlVal (panel, PANEL_ROWBYTES, rowBytes);
    SetCtrlVal (panel, PANEL_DEPTH, depth);
    SetCtrlVal (panel, PANEL_WIDTH, width);
    SetCtrlVal (panel, PANEL_HEIGHT, height);
    SetCtrlVal (panel, PANEL_COLORSIZE, colorTableSize);
    SetCtrlVal (panel, PANEL_BITMAPSIZE, bitmapSize);
    SetCtrlVal (panel, PANEL_MASKSIZE, maskSize);
    
    /* If the depth is > 8 bits, there is no colortable for the
     * image, so we dimm the controls. This program is not very useful
     * in such cases. To edit the colors your display must be set to
     * 8-bit depth or less.
     */
    if(colorTableSize == 0) {
        SetCtrlAttribute (panelHandle, PANEL_COLORTABLEINDEX, ATTR_DIMMED, 1);
        SetCtrlAttribute (panelHandle, PANEL_COLORTABLEENTRY, ATTR_DIMMED, 1);
        SetCtrlAttribute (panelHandle, PANEL_UPDATECOLORTABLE, ATTR_DIMMED, 1);
        SetCtrlAttribute (panelHandle, PANEL_TEXTMSG_2, ATTR_DIMMED, 1);
        return;
    }
    
    ClearListCtrl (panel, PANEL_COLORTABLEINDEX);
    numEntries = pow (2, depth);
    for (i = 0; i < numEntries; i++) {
        sprintf(istr, "%d", i);
        InsertListItem (panel, PANEL_COLORTABLEINDEX, -1, istr, i);
        }
    SetCtrlIndex (panel, PANEL_COLORTABLEINDEX, 0);
    ColorTableIndexCallback (panel, PANEL_COLORTABLEINDEX, EVENT_COMMIT, 0, 0, 0);
    
    free (colorTable);
    free (bitmap);
    free (mask);
    
    return;
}
