/*****************************************************************************/
/* This program demonstrates the use of intensity plots on a graph. By       */
/* changing the values in the color and data colors, different combinations  */
/* of the PlotIntensity() function parameters can be tried:                  */
/*                                                                           */
/* Color Settings:                                                           */
/*                                                                           */
/*  # of colors -   Ignored when color interpolation is enabled. This        */
/*                  determines the size of the color map array passed to     */
/*                  PlotIntensity(). This array is actually one entry        */
/*                  smaller than the value specified, since the color of the */
/*                  last entry is instead used for the Hi Color parameter.   */
/*  Low/High color -    These determine the range of colors which make up    */
/*                      the color map array. In this sample program, the     */
/*                      table is built by interpolating the RGB values       */
/*                      between these two colors. The data values in the     */
/*                      array are also interpolated from the Min/Max value   */
/*                      settings.                                            */
/*  Interpolate -   This controls the Interp Colors parameter of             */
/*                  PlotIntensity(). When enbaled, it allows that function   */
/*                  to perform the interpolation described above. In this    */
/*                  program, the array passed is only a minimal 2 entry      */
/*                  array consisting of the Low color/Low value entry and    */
/*                  the High Color/High value entry. The Hi Color parameter  */
/*                  in this case is not meaningful, since the array itself   */
/*                  contains the full range of the data.                     */
/*                                                                           */
/* Data Settings:                                                            */
/*                                                                           */
/*  # of X/Y elements - These determine the size of the data array used.     */
/*                      When data interpolation is disabled, they are the    */
/*                      main determinant of how long it takes to plot the    */
/*                      array. Keep in mind that increasing the array size   */
/*                      from 100 * 100 ( = 10000 points) to 140 x 140        */
/*                      ( = 19600 points), for example, will result not in a */
/*                      40% increase in time, but actually in close to a     */
/*                      100% increase.                                       */
/*  Min/Max value -     These determine the range of the values in the data  */
/*                      array. In this program, these are actually somewhat  */
/*                      meaningless, since this range is tied to the values  */
/*                      used in the color map, which means thet the colors   */
/*                      shown will not change by changing these values.      */
/* Interpolate -    This controls the Interp Pixels parameter of             */
/*                  PlotIntensity(). When enabled, every pixel within the    */
/*                  rectangular area of the plot will be interpolated from   */
/*                  the values of the 4 points which map to the nearest      */
/*                  pixels. In this case, the plotting time is determined    */
/*                  not by the size of the data array, but by the size in    */
/*                  pixels of the plot, which since this sample uses an      */
/*                  autoscaled graph, corresponds to the full size of the    */
/*                  plot area of the graph.                                  */
/*                                                                           */
/* The data used for plotting is generated while the program is waiting for  */
/* input, one row at a time inside a timer callback, so that plotting can be */
/* immediate once the Plot key is hit. Because of this, a horizontal line of */
/* discontinuity will be visible in the plots; it marks the row which was    */
/* being generated when the data was plotted.                                */
/*****************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "toolbox.h"
#include "intgraph.h"

#define MAXCOLORS   256

static void CoerceData (double *val, double minVal, double maxVal);
static int  GetParams (int panel, ColorMapEntry colors[], int *hiColor,
                int *interp, int *xSize, int *ySize, int *numColors);

static double   *data = NULL;

int main (int argc, char *argv[])
{
    double  lastMin, lastMax;
    int     i, panel, dialog, numX, numY, error = UIENoError;

    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    errChk(panel = LoadPanel (0, "intgraph.uir", PANEL));

    errChk(GetCtrlVal (panel, PANEL_XSIZE, &numX));
    errChk(GetCtrlVal (panel, PANEL_YSIZE, &numY));
    errChk(GetCtrlVal (panel, PANEL_MINVAL, &lastMin));
    errChk(GetCtrlVal (panel, PANEL_MAXVAL, &lastMax));
    errChk(SetCtrlAttribute (panel, PANEL_MINVAL, ATTR_CALLBACK_DATA, &lastMin));
    errChk(SetCtrlAttribute (panel, PANEL_MAXVAL, ATTR_CALLBACK_DATA, &lastMax));
    errChk(SetCtrlAttribute (panel, PANEL_TIMER, ATTR_CALLBACK_DATA, (void *)TRUE));
    errChk(SetCtrlAttribute (panel, PANEL_SCALE, ATTR_MAX_VALUE, numY - 1));
    
    srand (time (NULL));
    nullChk(data = (double *)calloc (numX * numY, sizeof (double)));
    
    errChk(DisplayPanel (panel));
    dialog = CreateProgressDialog ("Progress Bar", "Generating data...", TRUE,
        VAL_NO_MARKERS, NULL);
    
    SetCtrlAttribute (panel, PANEL_TIMER, ATTR_ENABLED, FALSE);
    for (i = 0; i < numY; i++) {
        UpdateProgressDialog (dialog, 100 * i / numY, FALSE);
        TimerCallback (panel, PANEL_TIMER, EVENT_TIMER_TICK, (void *)FALSE, 0, 0);
        }
    SetCtrlAttribute (panel, PANEL_TIMER, ATTR_ENABLED, TRUE);
    
    DiscardProgressDialog (dialog);
    errChk(RunUserInterface());
    
Error:
    if (data)
        free ((void *)data);

    return error;
}


int CVICALLBACK QuitCallback (int panel, int control, int event, void *callbackData,
    int eventData1, int eventData2)
{
    int error = UIENoError;
    
    switch (event) {
        case EVENT_COMMIT:
            errChk(QuitUserInterface (UIENoError));
            break;
        }
        
Error:
    return FALSE;
}


int CVICALLBACK SizeCallback (int panel, int control, int event, void *callbackData,
    int eventData1, int eventData2)
{
    int numX, numY, error = UIENoError;
    
    switch (event) {
        case EVENT_VAL_CHANGED:
            errChk(GetCtrlVal (panel, PANEL_XSIZE, &numX));
            errChk(GetCtrlVal (panel, PANEL_YSIZE, &numY));
            if (data)
                free ((void *)data);
            nullChk(data = (double *)calloc (numX * numY, sizeof (double)));
            errChk(SetCtrlAttribute (panel, PANEL_SCALE, ATTR_MAX_VALUE, numY - 1));
            break;
        }
Error:
    if (error != UIENoError)
        QuitUserInterface (error);
    return FALSE;
}

#define offset(x)   (double)rand() / RAND_MAX * x - x / 2.0;

int CVICALLBACK TimerCallback (int panel, int control, int event, void *callbackData,
    int eventData1, int eventData2)
{
    static int  row = -1;
    double      minVal, maxVal;
    int         range, i, numX, numY, error = UIENoError, updateScale;
    
    if (event != EVENT_TIMER_TICK)
        return FALSE;

    errChk(GetCtrlVal (panel, PANEL_XSIZE, &numX));
    errChk(GetCtrlVal (panel, PANEL_YSIZE, &numY));
    errChk(GetCtrlVal (panel, PANEL_MINVAL, &minVal));
    errChk(GetCtrlVal (panel, PANEL_MAXVAL, &maxVal));

    range = (maxVal - minVal) / 5;
    if (row == -1 || row >= numY) {     /* first row is generated differently */
        data[0] = (maxVal - minVal) / 2.0 + offset(range);
        for (i = 1; i < numX; i++) {
            data[i] = data[i-1] + offset(range);
            CoerceData (&data[i], minVal, maxVal);
            }
        row = 1;
        goto Error;
        }
    
    data[row*numX] = (data[(row-1)*numX] + data[(row-1)*numX+1]) / 2.0 +
        offset(range);
    CoerceData (&data[row*numX], minVal, maxVal);
    for (i = 1; i < numX - 1; i++) {
        data[row*numX+i] = (data[row*numX+i-1] + data[(row-1)*numX+i-1] +
            data[(row-1)*numX+i] + data[(row-1)*numX+i+1]) / 4.0 + offset(range);
        CoerceData (&data[row*numX+i], minVal, maxVal);
        }
    data[(row+1)*numX-1] = (data[(row+1)*numX-2] + data[row*numX-2] +
        data[row*numX-1]) / 3.0 + offset(range);
    CoerceData (&data[(row+1)*numX-1], minVal, maxVal);
    row++;

Error:
    updateScale = (int)callbackData;
    if (updateScale)
        SetCtrlVal (panel, PANEL_SCALE, row - 1);
    return FALSE;
}


int CVICALLBACK PlotCallback (int panel, int control, int event, void *callbackData,
    int eventData1, int eventData2)
{
    ColorMapEntry   colors[MAXCOLORS];  
    double          ygain, yoffset, xgain, xoffset;
    int             hiColor, numX, numY, numColors, interpColors, interpData,
                    error = UIENoError;
    
    switch (event) {
        case EVENT_COMMIT:
            errChk(GetParams (panel, colors, &hiColor, &interpColors, &numX,
                &numY, &numColors));
            errChk(DeleteGraphPlot (panel, PANEL_GRAPH, -1, VAL_DELAYED_DRAW));
            errChk(GetCtrlVal (panel, PANEL_INTERPDATA, &interpData));
            errChk(GetCtrlVal (panel, PANEL_YGAIN, &ygain));
            errChk(GetCtrlVal (panel, PANEL_YOFFSET, &yoffset));
            errChk(GetCtrlVal (panel, PANEL_XGAIN, &xgain));
            errChk(GetCtrlVal (panel, PANEL_XOFFSET, &xoffset));
            errChk(SetWaitCursor (TRUE));
            errChk(PlotScaledIntensity (panel, PANEL_GRAPH, (void *)data, numX, numY,
                VAL_DOUBLE, ygain, yoffset, xgain, xoffset, colors, hiColor, numColors,
                interpColors, interpData));
            errChk(SetWaitCursor (FALSE));
            break;
        }
    
Error:
    return FALSE;
}


int CVICALLBACK MinCallback (int panel, int control, int event, void *callbackData,
    int eventData1, int eventData2)
{
    double  newMin, max, *oldMinPtr;
    
    switch (event) {
        case EVENT_COMMIT:
            oldMinPtr = (double *)callbackData;
            GetCtrlVal (panel, control, &newMin);
            GetCtrlVal (panel, PANEL_MAXVAL, &max);
            if (newMin >= max) {
                MessagePopup ("Warning", "Minimum value must be smaller than maximum value");
                SetCtrlVal (panel, control, *oldMinPtr);
                }
            else
                *oldMinPtr = newMin;
            break;
        }
    return 0;
}


int CVICALLBACK MaxCallback (int panel, int control, int event, void *callbackData,
    int eventData1, int eventData2)
{
    double  newMax, min, *oldMaxPtr;
    
    switch (event) {
        case EVENT_COMMIT:
            oldMaxPtr = (double *)callbackData;
            GetCtrlVal (panel, control, &newMax);
            GetCtrlVal (panel, PANEL_MINVAL, &min);
            if (newMax <= min) {
                MessagePopup ("Warning", "Maximum value must be greater than minimum value");
                SetCtrlVal (panel, control, *oldMaxPtr);
                }
            else
                *oldMaxPtr = newMax;
            break;
        }
    return 0;
}


static void CoerceData (double *val, double minVal, double maxVal)
{
    if (*val < minVal)
        *val = minVal;
    if (*val > maxVal)
        *val = maxVal;

    return;
}

static int GetParams (int panel, ColorMapEntry colors[], int *hiColor,
    int *interpolate, int *xSize, int *ySize, int *numColors)
{
    unsigned char   loRed, hiRed, red, loGreen, hiGreen, green, loBlue, hiBlue,
                    blue;
    unsigned int    seed;
    double          minVal, maxVal;
    int             loCol, hiCol, i, j, error = UIENoError;

    errChk(GetCtrlVal (panel, PANEL_INTERPCOLORS, interpolate));
    errChk(GetCtrlVal (panel, PANEL_XSIZE, xSize));
    errChk(GetCtrlVal (panel, PANEL_YSIZE, ySize));
    errChk(GetCtrlVal (panel, PANEL_NUMCOL, numColors));
    errChk(GetCtrlVal (panel, PANEL_LOCOL, &loCol));
    errChk(GetCtrlVal (panel, PANEL_HICOL, &hiCol));
    errChk(GetCtrlVal (panel, PANEL_MINVAL, &minVal));
    errChk(GetCtrlVal (panel, PANEL_MAXVAL, &maxVal));

    if (*interpolate) {
        colors[0].color = loCol;
        colors[0].dataValue.valDouble = minVal;
        colors[1].color = hiCol;
        colors[1].dataValue.valDouble = maxVal;
        *numColors = 2;
        *hiColor = VAL_BLACK;
        }
    else {
        (*numColors)--;
        
        loRed = (unsigned char)((loCol & 0x00FF0000) >> 16);
        hiRed = (unsigned char)((hiCol & 0x00FF0000) >> 16);
        loGreen = (unsigned char)((loCol & 0x0000FF00) >> 8);
        hiGreen = (unsigned char)((hiCol & 0x0000FF00) >> 8);
        loBlue = (unsigned char)(loCol & 0x000000FF);
        hiBlue = (unsigned char)(hiCol & 0x000000FF);

        for (i = 0; i < *numColors; i++) {
            red = loRed + i * (hiRed - loRed) / *numColors;
            green = loGreen + i * (hiGreen - loGreen) / *numColors;
            blue = loBlue + i * (hiBlue - loBlue) / *numColors;
            colors[i].color = MakeColor ((int)red, (int)green, (int)blue);
            colors[i].dataValue.valDouble = minVal + (i + 1) * (maxVal - minVal)
                / (*numColors + 1);
            }
        *hiColor = MakeColor ((int)hiRed, (int)hiGreen, (int)hiBlue);
        }
        
Error:
    return error;
}


