/*
    PROGRAM:    physdemo.c

    PURPOSE:    This program is an demonstration of how to access physical memory from
                CVI. The two utility library functions are:

                int WriteToPhysicalMemory(unsigned address, char *dataPtr,
                                          unsigned short numBytes);
                int ReadFromPhysicalMemory(unsigned address, char *dataPtr,
                                           unsigned short numBytes);
*/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <ansi_c.h>
#include <userint.h>
#include <utility.h>
#include "physdemo.h"       /* header for physdemo.uir */


    /*** Macros: ***/

        /* monochrome display adapter characteristics: */
#define NUM_ROWS_IN_DISPLAY             24
#define NUM_COLUMNS_IN_DISPLAY          80
#define NUM_BYTES_PER_DISPLAY_CHARACTER 2
#define NUM_BYTES_PER_ROW               (NUM_BYTES_PER_DISPLAY_CHARACTER * NUM_COLUMNS_IN_DISPLAY)
#define NUM_BYTES_IN_DISPLAY (NUM_ROWS_IN_DISPLAY * NUM_BYTES_PER_ROW)
#define MONOCHROME_MEM_BASE_ADDRESS     0xB0000

    /*** Function Prototypes: ***/

static void CopyMonochromeMemoryByteArrayToTextBox(int panel, int textBox, char *array);
static void CopyTextBoxToMonochromeMemoryByteArray(int panel, int textBox, char *array);
static int  MonochromeAdapterMemoryPresent(void);
static void ShowProgramExplanation(void);


    /*** Global Variables: ***/

char    monochromeBuffer[NUM_BYTES_IN_DISPLAY];


    /*** Functions: ***/

int main (int argc, char *argv[])
{
int     panel;

    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */

    panel = LoadPanel(0, "physdemo.uir", PANEL);
    ShowProgramExplanation();
    DisplayPanel(panel);

    RunUserInterface();     /* will run until user presses the quit button */
    
    return 0;
}

/*******************************************************************************************/

static void ShowProgramExplanation(void)
{
    MessagePopup("Physical Memory Access Demo",
        "This is an example of how to access physical memory from CVI.\n\n"
        "The two Utility Library functions used are:\n"
        "   int WriteToPhysicalMemory(unsigned address, char *dataPtr, \n"
        "                           unsigned short numBytes);\n"
        "   int ReadFromPhysicalMemory(unsigned address, char *dataPtr,\n"
        "                            unsigned short numBytes);\n\n");
}

/*******************************************************************************************/

int CVICALLBACK CopyFromMonoDisplayCallback(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        {
        ReadFromPhysicalMemory(MONOCHROME_MEM_BASE_ADDRESS, monochromeBuffer, NUM_BYTES_IN_DISPLAY);
        CopyMonochromeMemoryByteArrayToTextBox(panel, PANEL_TEXTIN, monochromeBuffer);
        }
    return 0;
}

/*******************************************************************************************/

int CVICALLBACK CopyToMonoDisplayCallback(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        {
        CopyTextBoxToMonochromeMemoryByteArray(panel, PANEL_TEXTOUT, monochromeBuffer);
        WriteToPhysicalMemory(MONOCHROME_MEM_BASE_ADDRESS, monochromeBuffer, NUM_BYTES_IN_DISPLAY);
        }
    return 0;
}

/*******************************************************************************************/

void CVICALLBACK QuitItemCallback(int menubar, int menuItem, void *callbackData, int panel)
{
    QuitUserInterface(0);   /* process QUIT item in the menu bar */
}

/*******************************************************************************************/

int CVICALLBACK PanelCallback(int panel, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_CLOSE)     /* process a click on the close box on the panel */
        QuitUserInterface(0);
    return 0;
}

/*******************************************************************************************/

int CVICALLBACK ScrollDownCallback(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        {
        ReadFromPhysicalMemory(MONOCHROME_MEM_BASE_ADDRESS, monochromeBuffer, NUM_BYTES_IN_DISPLAY);
        memmove(&monochromeBuffer[NUM_BYTES_PER_ROW], monochromeBuffer, NUM_BYTES_IN_DISPLAY - NUM_BYTES_PER_ROW);
        memset(monochromeBuffer, ' ', NUM_BYTES_PER_ROW);
        WriteToPhysicalMemory(MONOCHROME_MEM_BASE_ADDRESS, monochromeBuffer, NUM_BYTES_IN_DISPLAY);
        }
    return 0;
}

/*******************************************************************************************/

int CVICALLBACK ScrollUpCallback(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        {
        ReadFromPhysicalMemory(MONOCHROME_MEM_BASE_ADDRESS, monochromeBuffer, NUM_BYTES_IN_DISPLAY);
        memmove(monochromeBuffer, &monochromeBuffer[NUM_BYTES_PER_ROW], NUM_BYTES_IN_DISPLAY - NUM_BYTES_PER_ROW);
        memset(&monochromeBuffer[NUM_BYTES_IN_DISPLAY - NUM_BYTES_PER_ROW -1], ' ', NUM_BYTES_PER_ROW);
        WriteToPhysicalMemory(MONOCHROME_MEM_BASE_ADDRESS, monochromeBuffer, NUM_BYTES_IN_DISPLAY);
        }
    return 0;
}

/****************************************************************************

   FUNCTION:    MonochromeAdapterMemoryPresent()
   PURPOSE:     Tests for the presence of monochrome display adapter memory
*/

static int MonochromeAdapterMemoryPresent(void)
{
char    savedByte[1], byteToWrite[1], byteReadBack[1];
int     index;
int     adapterPresent = 1;

    ReadFromPhysicalMemory(MONOCHROME_MEM_BASE_ADDRESS, savedByte, 1);
    for (index = 0; index <= 255; index++)
        {
        byteToWrite[0] = index;
        WriteToPhysicalMemory(MONOCHROME_MEM_BASE_ADDRESS, byteToWrite, 1);
        ReadFromPhysicalMemory(MONOCHROME_MEM_BASE_ADDRESS, byteReadBack, 1);

        if (byteReadBack[0] != byteToWrite[0])
            {
            adapterPresent = 0;
            break;
            }
        }

    WriteToPhysicalMemory(MONOCHROME_MEM_BASE_ADDRESS, savedByte, 1);
    return adapterPresent;
}

/*******************************************************************************************/

int CVICALLBACK CheckAdapterMemoryCallback(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        if(MonochromeAdapterMemoryPresent())
            MessagePopup("Monochrome Display Adapter Memory Check", "Display adapter memory was found at B000:0000");
        else
            MessagePopup("Monochrome Display Adapter Memory Check", "Error:\n\nNo display adapter memory was found at B000:0000\nThis demo will not function properly.");
    return 0;
}

/*******************************************************************************************/

static void CopyMonochromeMemoryByteArrayToTextBox(int panel, int textBox, char *array)
{
char    lineBuf[NUM_COLUMNS_IN_DISPLAY+2];  /* +2 == 1 byte for '\n' and 1 byte for null terminator */
int     arrayIndex=0, lineBufIndex = 0, line = 0;
int     numBytesLeft = NUM_BYTES_IN_DISPLAY;

    ResetTextBox (panel, textBox, "");
    while (numBytesLeft > 0)
        {
        lineBuf[lineBufIndex] = array[arrayIndex];
        lineBufIndex++;
        arrayIndex += 2;        /* skip over text attribute bytes */
        numBytesLeft -= 2;

        if (lineBufIndex == NUM_COLUMNS_IN_DISPLAY)
            {
            lineBuf[NUM_COLUMNS_IN_DISPLAY] = 0;    /* NULL terminator */
            if (line != NUM_ROWS_IN_DISPLAY-1)
                InsertTextBoxLine(panel, textBox, line, lineBuf);
            else
                SetCtrlVal (panel, textBox, lineBuf);
            lineBufIndex = 0;
            line++;
            }
        }
    SetCtrlAttribute (panel, textBox, ATTR_HSCROLL_OFFSET, 0);
    SetCtrlAttribute (panel, textBox, ATTR_FIRST_VISIBLE_LINE, 0);
}

/*******************************************************************************************/

static void CopyTextBoxToMonochromeMemoryByteArray(int panel, int textBox, char *array)
{
char    lineBuf[256];
int     arrayIndex=0;
int     lineBufIndex = 0;
int     line = 0;
int     lineLength;
int     numBytesLeft = NUM_BYTES_IN_DISPLAY;
int     numTextBoxLines;

    GetNumTextBoxLines (panel, textBox, &numTextBoxLines);

    while (line < NUM_ROWS_IN_DISPLAY)
        {
        if (line < numTextBoxLines)
            {
            GetTextBoxLineLength (panel, textBox, line, &lineLength);
            if (lineLength >= 255)
                {
                strcpy(lineBuf, "<text box line too long!!!>");
                lineLength = strlen(lineBuf);
                }
            else
                GetTextBoxLine (panel, textBox, line, lineBuf);
            }
        else
            lineLength = 0;

        for (lineBufIndex = 0; lineBufIndex < NUM_COLUMNS_IN_DISPLAY; lineBufIndex++)
            {
            if (lineBufIndex < lineLength)
                array[arrayIndex] = lineBuf[lineBufIndex];
            else
                array[arrayIndex] = 0;

            arrayIndex++;
            array[arrayIndex] = 7;  /* character attribute byte */
            arrayIndex++;
            }
        line++;
        }
}
